// i2c.cpp : Diese Datei enthält die Funktion "main". Hier beginnt und endet die Ausführung des Programms.
//

#include <iostream>
#include <Windows.h>
#include <iomanip>
#include "iowkit.h"

int main()
{
	IOWKIT_HANDLE handle;
	IOWKIT100_SPECIAL_REPORT report;
	int num = 0;
	handle = IowKitOpenDevice();

	if (handle != NULL)
	{
		//Only continue if IO-Warrior100 found
		if(IowKitGetProductId(handle) == IOWKIT_PID_IOW100)
		{
			//Enable I2C mode
			memset(&report, 0x00, IOWKIT100_SPECIAL_REPORT_SIZE);
			report.ReportID = 0x01;	//Special mode I2C
			report.Bytes[0] = 0x01;	//1: Enable, 0: Disable
			report.Bytes[1] = 0x00; //flags

			IowKitWrite(handle, IOW_PIPE_I2C_MODE, (char*)&report, IOWKIT100_SPECIAL_REPORT_SIZE);
			//No IowKitRead() possible for setting special mode

			//Search for I2C devices on bus
			for (int i = 1; i < 127; i++)
			{
				memset(&report, 0, IOWKIT100_SPECIAL_REPORT_SIZE);
				report.ReportID = 0x03;		//Read
				report.Bytes[0] = 0x01;		//count
				report.Bytes[1] = (i << 1) | 0x01;	//8bit I2C address + read bit
				report.Bytes[2] = 0x00;		//Mcount

				if (IowKitWrite(handle, IOW_PIPE_I2C_MODE, (char*)&report, IOWKIT100_SPECIAL_REPORT_SIZE) == IOWKIT100_SPECIAL_REPORT_SIZE)
				{
					if (IowKitRead(handle, IOW_PIPE_I2C_MODE, (char*)&report, IOWKIT100_SPECIAL_REPORT_SIZE) == IOWKIT100_SPECIAL_REPORT_SIZE)
					{
						if (report.Bytes[0] == 0x01) //Read byte 0, if byte value = 1 -> device present
						{
							std::cout << "I2C device found (7bit): 0x" << std::setfill('0') << std::setw(2) << std::hex << (i & 0xFF) << std::dec << std::endl;
							num++;
						}
					}
					else
					{
						std::cout << "Error: Fail to read from IO-Warrior100 (IowKitRead())" << std::endl;
						break;
					}
				}
				else
				{
					std::cout << "Error: Fail to write IO-Warrior100 (IowKitWrite())" << std::endl;
					break;
				}
			}

			std::cout << "Devices found on I2C bus: " << num << std::endl;


			//Disable I2C mode (optional)
			memset(&report, 0, IOWKIT100_SPECIAL_REPORT_SIZE);
			report.ReportID = 0x01;	//Special mode I2C
			report.Bytes[0] = 0x00;	//1: Enable, 0: Disable
			report.Bytes[1] = 0x00; //flags

			IowKitWrite(handle, IOW_PIPE_I2C_MODE, (char*)&report, IOWKIT100_SPECIAL_REPORT_SIZE);
			//No IowKitRead() possible for setting special mode
		}
		else
		{
			std::cout << "No IO-Warrior100 found" << std::endl;
		}
	}
	else
	{
		std::cout << "No IO-Warrior found" << std::endl;
	}

	IowKitCloseDevice(handle); //Don't forget to close
}

